@@ -25,25 +25,29 @@ select_opts = [] # Will hold all the selection options
 select_files = [] # Will hold file objects when filelist given
 full_backup = None # Will be set to true if -f or --full option given
 list_current = None # Will be set to true if --list-current option given
+collection_status = None # Will be set to true if --collection-status given
+cleanup = None # Set to true if --cleanup option given
 
 def parse_cmdline_options(arglist):
 	"""Parse argument list"""
-	global select_opts, select_files, full_backup, list_current
+	global select_opts, select_files, full_backup
+	global list_current, collection_status, cleanup, remove_time
 	def sel_fl(filename):
 		"""Helper function for including/excluding filelists below"""
 		try: return open(filename, "r")
 		except IOError: log.FatalError("Error opening file %s" % filename)
 
 	try: optlist, args = getopt.getopt(arglist, "firt:v:V",
-		 ["allow-source-mismatch", "archive-dir=", "current-time=",
-		  "encrypt-key=", "exclude=", "exclude-device-files",
-		  "exclude-filelist=", "exclude-globbing-filelist",
-		  "exclude-filelist-stdin", "exclude-other-filesystems",
-		  "exclude-regexp=", "file-to-restore=", "full",
-		  "incremental", "include=", "include-filelist=",
-		  "include-filelist-stdin", "include-globbing-filelist=",
-		  "include-regexp=", "list-current-files",
-		  "no-print-statistics", "null-separator", "restore-dir=",
+		 ["allow-source-mismatch", "archive-dir=", "cleanup",
+		  "current-time=", "collection-status", "encrypt-key=",
+		  "exclude=", "exclude-device-files", "exclude-filelist=",
+		  "exclude-globbing-filelist", "exclude-filelist-stdin",
+		  "exclude-other-filesystems", "exclude-regexp=",
+		  "file-to-restore=", "force", "full", "incremental",
+		  "include=", "include-filelist=", "include-filelist-stdin",
+		  "include-globbing-filelist=", "include-regexp=",
+		  "list-current-files", "no-print-statistics",
+		  "null-separator", "remove-older-than=", "restore-dir=",
 		  "restore-time=", "scp-command=", "short-filenames",
 		  "sign-key=", "ssh-command=", "verbosity="])
 	except getopt.error, e:
@@ -52,8 +56,10 @@ def parse_cmdline_options(arglist):
 	for opt, arg in optlist:
 		if opt == "--allow-source-mismatch": globals.allow_source_mismatch = 1
 		elif opt == "--archive-dir": set_archive_dir(arg)
+		elif opt == "--cleanup": cleanup = 1
+		elif opt == "--collection-status": collection_status = 1
 		elif opt == "--current-time":
-			globals.current_time = get_int(arg, "current-time")
+			dup_time.setcurtime(get_int(arg, "current-time"))
 		elif opt == "--encrypt-key":
 			globals.gpg_profile.recipients.append(arg)
 		elif (opt == "--exclude" or opt == "--exclude-regexp" or
@@ -71,6 +77,7 @@ def parse_cmdline_options(arglist):
 			select_opts.append(("--exclude-filelist", "standard input"))
 			select_files.append(sys.stdin)
 		elif opt == "-f" or opt == "--full": full_backup = 1
+		elif opt == "--force": globals.force = 1
 		elif opt == "--include-filelist-stdin":
 			select_opts.append(("--include-filelist", "standard input"))
 			select_files.append(sys.stdin)
@@ -80,6 +87,8 @@ def parse_cmdline_options(arglist):
 		elif opt == "--null-separator": globals.null_separator = 1
 		elif opt == "-r" or opt == "--file-to-restore":
 			globals.restore_dir = arg
+		elif opt == "--remove-older-than":
+			globals.remove_time = dup_time.genstrtotime(arg)
 		elif opt == "-t" or opt == "--restore-time":
 			globals.restore_time = dup_time.genstrtotime(arg)
 		elif opt == "--scp-command": backends.scp_command = arg
@@ -168,8 +177,16 @@ def process_local_dir(action, local_pathname):
 def check_consistency(action):
 	"""Final consistency check, see if something wrong with command line"""
 	global full_backup, select_opts, list_current
-	if action == "list-current":
-		if not list_current: command_line_error("Too few arguments")
+	def assert_only_one(arglist):
+		"""Raises error if two or more of the elements of arglist are true"""
+		n = 0
+		for m in arglist:
+			if m: n+=1
+		assert n <= 1, "Invalid syntax, two conflicting modes specified"
+	if action in ["list-current", "collection-status",
+				  "cleanup", "remove-old"]:
+		assert_only_one([list_current, collection_status, cleanup,
+						 globals.remove_time is not None])
 	elif action == "restore":
 		if full_backup:
 			command_line_error("--full option cannot be used when restoring")
@@ -192,14 +209,16 @@ def ProcessCommandLine(cmdline_list):
 	action will be "list-current", "restore", "full", or "inc".
 
 	"""
-	global full_backup
 	globals.gpg_profile = gpg.GPGProfile()
 
 	args = parse_cmdline_options(cmdline_list)
 	if len(args) < 1: command_line_error("Too few arguments")
 	elif len(args) == 1:
-		if not list_current: command_line_error("Too few arguments")
-		action = "list-current"
+		if list_current: action = "list-current"
+		elif collection_status: action = "collection-status"
+		elif cleanup: action = "cleanup"
+		elif globals.remove_time is not None: action = "remove-old"
+		else: command_line_error("Too few arguments")
 		globals.backend = backends.get_backend(args[0])
 		if not globals.backend: log.FatalError("""Bad URL '%s'.
 Examples of URL strings are "scp://user@host.net:1234/path" and
@@ -216,4 +235,5 @@ Examples of URL strings are "scp://user@host.net:1234/path" and
 	elif len(args) > 2: command_line_error("Too many arguments")
 
 	check_consistency(action)
+	log.Log("Main action: " + action, 7)
 	return action
